"use strict";
/**
 * SPDX-FileCopyrightText: © 2020 Liferay, Inc. <https://liferay.com>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeChunks = exports.flattenResponses = exports.createTranslationsObject = exports.arrayToMap = exports.addMissingTranslations = void 0;
/**
 * © 2017 Liferay, Inc. <https://liferay.com>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */
const js_toolkit_core_1 = require("@liferay/js-toolkit-core");
const fs_1 = __importDefault(require("fs"));
const properties_1 = __importDefault(require("properties"));
const request_1 = __importDefault(require("request"));
const v4_1 = __importDefault(require("uuid/v4"));
const config_1 = require("../config");
const { debug, error, info, print, success, warn } = js_toolkit_core_1.format;
/**
 * Default entry point
 */
function default_1() {
    if (!config_1.project.l10n.supported) {
        print(error `
			Project does not support localization: nothing to translate.
		`);
        process.exit(3);
    }
    let subscriptionKey = process.env.TRANSLATOR_TEXT_KEY;
    if (!subscriptionKey) {
        subscriptionKey = config_1.getTranslatorTextKey();
    }
    if (!subscriptionKey || subscriptionKey === '') {
        print(error `
			-------------------------------------------------------------
			🛑 Microsoft Translator credentials not set 🛑

			Please set the translatorTextKey variable in your .npmbuildrc
			file or provide them in the TRANSLATOR_TEXT_KEY environment
			variable.
			-------------------------------------------------------------
		`);
        process.exit(2);
    }
    showMissingSupportedLocales();
    createMissingSupportedLocalesFiles();
    const localizationFileMap = config_1.project.l10n.localizationFileMap;
    const locales = Object.keys(localizationFileMap).filter((locale) => locale != 'default');
    if (locales.length === 0) {
        print(error `No locales found: nothing to translate`);
        print(info `
			You can edit your .npmbuildrc file to add new supported locales
			using the 'supportedLocales' array.
		`);
        return;
    }
    print(debug `Found ${locales.length} locales: ${locales.join(', ')}`);
    print(info `Adding missing translations:`);
    Promise.all([
        translateFile(subscriptionKey, locales, localizationFileMap.default.asNative),
        ...locales.map((locale) => parseFile(localizationFileMap[locale].asNative)),
    ])
        .then(([translation, ...labels]) => addMissingTranslations(translation, arrayToMap(labels, locales)))
        .then((labels) => {
        print(debug `Writing localization files:`);
        Object.entries(labels).forEach(([locale, labels]) => {
            fs_1.default.writeFileSync(localizationFileMap[locale].asNative, properties_1.default.stringify(labels));
            print(debug `  · Wrote ${localizationFileMap[locale].asNative}`);
        });
        print(success `Finished`);
    })
        .catch((err) => {
        print(error `
				There was an error translating files:

				  ${err}	
				
			`);
        process.exit(1);
    });
}
exports.default = default_1;
/**
 * Adds missing translations from the translation parameter to the labels
 * parameter.
 * @param {object} translation map of label maps indexed by locale
 * @param {object} labels map of label maps indexed by locale
 * @return {object} the modified labels parameter
 */
function addMissingTranslations(translation, labels) {
    const locales = Object.keys(translation);
    locales.forEach((locale) => {
        const map = translation[locale];
        let count = 0;
        Object.entries(map).forEach(([key, value]) => {
            if (!labels[locale][key]) {
                labels[locale][key] = value;
                count++;
            }
        });
        print(count === 0
            ? info `  · No missing translations found for locale ${locale}`
            : info `  · Added ${count} missing translations for locale ${locale}`);
    });
    return labels;
}
exports.addMissingTranslations = addMissingTranslations;
/**
 * Converts an array of items into a map with given keys as property names.
 * @param {Array<*>} array array of values
 * @param {Array<string>} keys array of key names
 * @return {object}
 */
function arrayToMap(array, keys) {
    return array.reduce((map, item, index) => ((map[keys[index]] = item), map), {});
}
exports.arrayToMap = arrayToMap;
/**
 *
 * @param {Array<string>} locales
 * @return {object} an object with empty arrays as properties, indexed by locale
 */
function createTranslationsObject(locales) {
    return locales.reduce((translations, locale) => ((translations[locale] = []), translations), {});
}
exports.createTranslationsObject = createTranslationsObject;
/**
 * Flatten an array or arrays
 * @param {Array<Array<object>>} responses
 * @return {Array<object>}
 */
function flattenResponses(responses) {
    return responses.reduce((flatResponses, response) => (flatResponses.push(...response), flatResponses), []);
}
exports.flattenResponses = flattenResponses;
/**
 * Split an array of strings into sub arrays of limited length, suitable for use
 * with Microsoft Translator API.
 * @param {Array<string>} texts
 * @return {Array<Array<string>>} split text arrays
 */
function makeChunks(texts) {
    const chunks = [];
    let currentChunk;
    let chars;
    // A helper to add a new chunk to the `chunks` array
    function appendChunk() {
        chars = 0;
        currentChunk = [];
        chunks.push(currentChunk);
    }
    appendChunk();
    texts.forEach((text) => {
        // Limit total request to 5000 chars, but we need room for []
        chars += text.length + 11; // 11 is for {"Text":},<CR>
        if (chars > 4098) {
            appendChunk();
        }
        // Limit total request texts to 100
        if (currentChunk.length >= 100) {
            appendChunk();
        }
        // Register request
        currentChunk.push(text);
    });
    return chunks;
}
exports.makeChunks = makeChunks;
/**
 * Show missing supported locales: those for which there's a .properties file
 * but are not configured.
 */
function showMissingSupportedLocales() {
    const availableLocales = config_1.project.l10n.availableLocales;
    const supportedLocales = config_1.getSupportedLocales();
    const missingLocales = availableLocales.filter((locale) => supportedLocales.indexOf(locale) === -1);
    if (missingLocales.length > 0) {
        print(warn `Found ${missingLocales.length} unsupported locale files:`);
        missingLocales.forEach((locale) => print(warn `  · Found file for unsupported locale ${locale}`));
        print(info `
			You can edit your .npmbuildrc file to add these unsupported
			locales or remove their .properties files to hide this warning.
		`);
    }
}
/**
 * Creates missing locale files according to .npmbuildrc configuration
 */
function createMissingSupportedLocalesFiles() {
    const supportedLocales = config_1.getSupportedLocales();
    const missingLocales = supportedLocales.filter((locale) => config_1.project.l10n.availableLocales.indexOf(locale) === -1);
    if (missingLocales.length > 0) {
        const languageFileBaseName = config_1.project.l10n.languageFileBaseName;
        missingLocales.forEach((locale) => fs_1.default.writeFileSync(`${languageFileBaseName.asNative}_${locale}.properties`, ''));
    }
}
/**
 * Parses a .properties file
 * @param {string} filePath
 * @return {Promise<object>} the map of properties
 */
function parseFile(filePath) {
    return new Promise((resolve, reject) => {
        properties_1.default.parse(filePath, { path: true }, (err, labels) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(labels);
            }
        });
    });
}
/**
 * @param {string} subscriptionKey
 * @param {Array<string>} locales
 * @param {Array<string>} texts
 * @return {Promise<object>} a map of arrays with translated texts indexed by
 * 								locale
 */
function translate(subscriptionKey, locales, texts) {
    // Map from ['es_ES', 'es_AR'] to {'es_ES': 'es', 'es_AR': 'es'}
    const localesMap = locales.reduce((map, locale) => {
        const targetLocale = locale.split('_')[0];
        if (map[targetLocale] === undefined) {
            map[targetLocale] = [];
        }
        map[targetLocale].push(locale);
        return map;
    }, {});
    const targetLocales = [...new Set(Object.keys(localesMap))];
    const options = {
        baseUrl: 'https://api.cognitive.microsofttranslator.com/',
        headers: {
            'Content-type': 'application/json',
            'Ocp-Apim-Subscription-Key': subscriptionKey,
            'X-ClientTraceId': v4_1.default().toString(),
        },
        json: true,
        method: 'POST',
        qs: {
            'api-version': '3.0',
            to: targetLocales,
        },
        url: 'translate',
    };
    const promises = makeChunks(texts).map((chunk) => new Promise((resolve, reject) => {
        request_1.default({
            ...options,
            body: chunk.map((item) => ({ text: item })),
        }, (err, response, body) => {
            if (err) {
                reject(err);
            }
            else if (response.statusCode != 200) {
                reject({
                    code: response.statusCode,
                    message: response.statusMessage,
                });
            }
            else if (body.error) {
                reject(body.error);
            }
            else {
                resolve(body);
            }
        });
    }));
    return Promise.all(promises).then((responses) => {
        const translations = createTranslationsObject(locales);
        flattenResponses(responses).forEach((response) => {
            response['translations'].forEach((translation) => {
                localesMap[translation.to].forEach((locale) => translations[locale].push(translation.text));
            });
        });
        return translations;
    });
}
/**
 *
 * @param {string} subscriptionKey
 * @param {Array<string>} locales
 * @param {string} filePath
 * @return {Promise<object>} map of label maps indexed by locale
 */
function translateFile(subscriptionKey, locales, filePath) {
    let keys;
    return parseFile(filePath)
        .then((labels) => {
        keys = Object.keys(labels);
        return translate(subscriptionKey, locales, keys.map((key) => labels[key]));
    })
        .then((translation) => {
        locales.forEach((locale) => {
            translation[locale] = arrayToMap(translation[locale], keys);
        });
        return translation;
    });
}
